@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011, 2012, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2011 – 2017 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,16 +31,7 @@
 G_BEGIN_DECLS
 
 #define GOA_TYPE_OAUTH_PROVIDER         (goa_oauth_provider_get_type ())
-#define GOA_OAUTH_PROVIDER(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOA_TYPE_OAUTH_PROVIDER, GoaOAuthProvider))
-#define GOA_OAUTH_PROVIDER_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST ((k), GOA_TYPE_OAUTH_PROVIDER, GoaOAuthProviderClass))
-#define GOA_OAUTH_PROVIDER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOA_TYPE_OAUTH_PROVIDER, GoaOAuthProviderClass))
-#define GOA_IS_OAUTH_PROVIDER(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GOA_TYPE_OAUTH_PROVIDER))
-
-#define GOA_IS_OAUTH_PROVIDER_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOA_TYPE_OAUTH_PROVIDER))
-
-typedef struct _GoaOAuthProvider GoaOAuthProvider;
-typedef struct _GoaOAuthProviderClass GoaOAuthProviderClass;
-typedef struct _GoaOAuthProviderPrivate GoaOAuthProviderPrivate;
+G_DECLARE_DERIVABLE_TYPE (GoaOAuthProvider, goa_oauth_provider, GOA, OAUTH_PROVIDER, GoaProvider);
 
 /**
  * GoaOAuthProvider:
@@ -48,12 +39,6 @@ typedef struct _GoaOAuthProviderPrivate GoaOAuthProviderPrivate;
  * The #GoaOAuthProvider structure contains only private data and should
  * only be accessed using the provided API.
  */
-struct _GoaOAuthProvider
-{
-  /*< private >*/
-  GoaProvider parent_instance;
-  GoaOAuthProviderPrivate *priv;
-};
 
 /**
  * GoaOAuthProviderClass:
@@ -119,14 +104,10 @@ struct _GoaOAuthProviderClass
   gboolean     (*is_password_node)             (GoaOAuthProvider             *provider,
                                                 WebKitDOMHTMLInputElement    *element);
 
-  /*< private >*/
-  GoaOAuthProviderPrivate *priv;
-
   /* Padding for future expansion */
-  gpointer goa_reserved[28];
+  gpointer goa_reserved[29];
 };
 
-GType        goa_oauth_provider_get_type                     (void) G_GNUC_CONST;
 const gchar *goa_oauth_provider_get_consumer_key             (GoaOAuthProvider             *provider);
 const gchar *goa_oauth_provider_get_consumer_secret          (GoaOAuthProvider             *provider);
 const gchar *goa_oauth_provider_get_request_uri              (GoaOAuthProvider             *provider);
