@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2013, 2015 Red Hat, Inc.
+ * Copyright © 2013 – 2017 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include "goaprovider.h"
 #include "goaprovider-priv.h"
 #include "goaoauth2provider.h"
+#include "goaoauth2provider-priv.h"
 #include "goapocketprovider.h"
 #include "goaobjectskeletonutils.h"
 
@@ -35,6 +36,8 @@ struct _GoaPocketProvider
 {
   GoaOAuth2Provider parent_instance;
 
+  gchar *authorization_uri;
+
   /* request token as gathered from Step 2:
    * http://getpocket.com/developer/docs/authentication */
   gchar *code;
@@ -127,14 +130,15 @@ build_authorization_uri (GoaOAuth2Provider  *oauth2_provider,
                          const gchar        *escaped_scope)
 {
   GoaPocketProvider *self = GOA_POCKET_PROVIDER (oauth2_provider);
-  RestProxy *proxy;
-  RestProxyCall *call;
+  RestProxy *proxy = NULL;
+  RestProxyCall *call = NULL;
   const gchar *payload;
   gchar *code, *url = NULL;
   GError *error = NULL;
   GHashTable *hash;
 
-  g_clear_pointer (&self->code, g_free);
+  if (self->authorization_uri != NULL)
+    goto end;
 
   proxy = rest_proxy_new (get_request_uri (oauth2_provider), FALSE);
   call = rest_proxy_new_call (proxy);
@@ -162,21 +166,50 @@ build_authorization_uri (GoaOAuth2Provider  *oauth2_provider,
       goto out;
     }
 
-  url = g_strdup_printf ("%s"
-                         "?request_token=%s"
-                         "&redirect_uri=%s",
-                         authorization_uri,
-                         code,
-                         escaped_redirect_uri);
+  self->authorization_uri = g_strdup_printf ("%s"
+                                             "?request_token=%s"
+                                             "&redirect_uri=%s",
+                                             authorization_uri,
+                                             code,
+                                             escaped_redirect_uri);
 
   self->code = code;
 
-out:
+ end:
+  url = g_strdup (self->authorization_uri);
+
+ out:
   g_clear_object (&call);
   g_clear_object (&proxy);
   return url;
 }
 
+static gboolean
+decide_navigation_policy (GoaOAuth2Provider               *oauth2_provider,
+                          WebKitWebView                   *web_view,
+                          WebKitNavigationPolicyDecision  *decision)
+{
+  GoaPocketProvider *self = GOA_POCKET_PROVIDER (oauth2_provider);
+  WebKitNavigationAction *action;
+  WebKitURIRequest *request;
+  gboolean ret = FALSE;
+  const gchar *uri;
+
+  action = webkit_navigation_policy_decision_get_navigation_action (decision);
+  request = webkit_navigation_action_get_request (action);
+  uri = webkit_uri_request_get_uri (request);
+  if (!g_str_has_prefix (uri, "https://getpocket.com/a/"))
+    goto out;
+
+  webkit_uri_request_set_uri (request, self->authorization_uri);
+  webkit_web_view_load_request (web_view, request);
+
+  ret = TRUE;
+
+ out:
+  return ret;
+}
+
 static gboolean
 process_redirect_url (GoaOAuth2Provider            *oauth2_provider,
                       const gchar                  *redirect_url,
@@ -378,6 +411,7 @@ goa_pocket_provider_finalize (GObject *object)
 {
   GoaPocketProvider *self = GOA_POCKET_PROVIDER (object);
 
+  g_free (self->authorization_uri);
   g_clear_pointer (&self->code, g_free);
   g_clear_pointer (&self->identity, g_free);
 
@@ -400,6 +434,7 @@ goa_pocket_provider_class_init (GoaPocketProviderClass *klass)
   provider_class->build_object               = build_object;
 
   oauth2_class->build_authorization_uri   = build_authorization_uri;
+  oauth2_class->decide_navigation_policy  = decide_navigation_policy;
   oauth2_class->get_authorization_uri     = get_authorization_uri;
   oauth2_class->get_token_uri             = get_token_uri;
   oauth2_class->get_redirect_uri          = get_redirect_uri;
