@@ -67,7 +67,7 @@ static gchar *
 get_provider_name (GoaProvider *provider,
                    GoaObject   *object)
 {
-  return g_strdup (_("Microsoft Account"));
+  return g_strdup (_("Microsoft"));
 }
 
 static GIcon *
